angular.module('myApp')

.controller('MainController', ['$scope', '$localStorage', 'mainService', function ($scope, $localStorage, mainService) {
    var main = this;
    main.nodes = mainService.nodes;
    main.bombs = mainService.bombs;
    main.flags = mainService.flags;
    main.nodeModels = mainService.nodeModels;
    main.counters = mainService.counters;
    main.boardApi = {};

    main.$storage = $localStorage.$default({
        scoreBoard : []
    });

    var selectedModel;

    main.addNode = function(pos) {
        if (!selectedModel) {return}
        var newNode = angular.copy(selectedModel);
        delete newNode.selected;
        newNode.pos = pos;
        mainService.addNode(newNode);
        $scope.$apply();
    }

    main.addNodeOfType = function(pos, type) {
        if (main.gameover||main.gamewin) {return};
        var model = main.nodeModels.find(function(m) {return m.type==type});
        var newNode = angular.copy(model);
        newNode.pos = pos;
        var explosion = mainService.addNode(newNode);
        if (explosion) {
            //console.log("EXPLOSION")
            endGame(false);
        }
        $scope.$apply();
    }

    main.selectModel = function(model) {
        selectedModel && (selectedModel.selected = false);
        model.selected = true;
        selectedModel = model;
    }

    main.removeFlag = function(flag) {
        if (main.gameover||main.gamewin) {return};
        mainService.removeFlag(flag);
        $scope.$apply();
    }

    /* Actually it just disable the probe */
    main.removeProbe = function(probe) {
        if (main.gameover||main.gamewin) {return};
        //mainService.removeProbe(probe);
        probe.disabled = !probe.disabled;
        $scope.$apply();
    }

    main.clearArea = function(probe) {
        if (main.gameover||main.gamewin) {return}
        if (probe.disabled) {return}
        if (!mainService.canClear(probe)) {return false};

        var explosion = mainService.clearArea(probe);
        if (explosion) {
            endGame(false);
        } else if (mainService.allclear) {
            endGame(true);
        }
        $scope.$apply();
    }

    main.showBombs = function() {
        main.boardApi.showBombs();
    }

    main.newGame = function() {
        main.boardApi.clear();
        init();
    }

    function endGame(win) {
        main.gamewin = win;
        main.gameover = !win;
        clearInterval(main.timer);

        if (win) {
            main.$storage.scoreBoard.push({t:main.time,p:main.counters.usedProbes});
        } else {
            main.showBombs();
        }
    }

    function init() {
        main.gameover = false;
        main.gamewin = false;
        main.time = 0;
        clearInterval(main.timer);
        main.timer = setInterval(function(){main.time++;$scope.$apply()}, 1000);
        mainService.init();
    }

    init();
}])

.filter('secondsTommss', function($filter) {
    return function(seconds) {
        return $filter('date')(new Date(0, 0, 0).setSeconds(seconds), 'mm:ss');
    };
})
;